--|  
--| Filename        : $Source: /home/ACDE/Work/MySQL/RCS/db.ads,v $
--| Description     : Template Specification
--| Author          : Michael Erdmann
--| Created On      : 25.3.1999
--| Last Modified By: $Author: ACDE $
--| Last Modified On: $Date: 1999/11/01 07:02:08 $
--| Status          : $State: Exp $
--|
--|
--| Functional Description
--| ======================
--| This component implements a thick binding to MySQL, which is 
--| available for Linix. This interface allows Ada 95 clients to communicate
--| with the MYSQL server at any host.
--|
--| Queries are sent to the server a sql strings. The result of a query
--| may be processes by the client row by row. Eeach query receives a 
--| Query id which allows to use multiple queries at the same time.
--|
--| The protocol between client and server is the following one:
--|
--|   Set User Id
--|   Connect to Server
--|   Select a data base
--|
--|   while Application is active 
--|      Issue a query
--|      while rows available 
--|          process data;
--|      drop Query
--|
--|   disconnect
--|
--|
--| Component Data
--| ==============
--| User  - Name of the User logged in
--|
--| Query_ID     - Id of a query. This resource is allocated by the Query command and
--|                can be released by the Drop_Query.
--| Field_Number - The colum number of a field in the query result.
--|
--| Operations on the component data
--| ================================
--| see component specification below and documentation.
--|
--| Error Handling
--| ==============
--| Error handling is done by exceptions. See documentattion.
--|
--|
--| Extension
--| =========
--| None
--|
--| Restrictions
--| ============
--| Upto 10 active queries per connections
--| Upto 100 field per query.
--|
--| History
--| =======
--| $Log: db.ads,v $
--| Revision 1.3  1999/11/01 07:02:08  ACDE
--| inline documentation updated, Error handling updated
--|
--| Revision 1.2  1999/10/30 15:50:49  ACDE
--| float fileds supported
--|
--| Revision 1.1  1999/10/17 15:17:50  ACDE
--| String fieldtypes are functioning. Other fields are missing.
--|
--|
with Ada.Strings.Unbounded; use Ada.Strings.Unbounded;
with Ada.Calendar;          use Ada.Calendar;

package DB is
   
   ---=====================================================================---
   ---===             C O M P O N E N T   I N T E R F A C E             ===---
   ---=====================================================================---
   type Object is tagged private;
   type Handle is access Object'Class;

   procedure Initialize( this  : in out Object'Class);
   procedure Finalize( this : in out Object'Class );

   Usage_Error      : exception;
   Not_Initialized  : exception;
   Out_Of_Resources : exception;

   type    Query_ID is new Positive range 1..10;

   subtype Field_Number is Integer range 1..100;

   subtype Blob is String;


   DB_Name_Failure  : exception;

   ---=====================================================================---
   ---===                  A T T R I B U T E S                       ===---
   ---=====================================================================---

   procedure User( this : in out Object'Class;  Name : in String );

   ---=====================================================================---
   ---===                     M E T H O D S                             ===---
   ---=====================================================================---

   ---------------------------------------------------------------------------
   --| Description    : Connect to the given server, use the user name which
   --|              	has been set by the User Attribute.
   --| Preconditions  : none.
   --| Postconditions : Server is connected
   --| Exceptions     :
   --| Note           :
   ---------------------------------------------------------------------------
   procedure Connect( this : in out Object'Class; Server : in String );

   Connect_Failure : exception;

   ---------------------------------------------------------------------------
   --| Description    : Disconnect from the server.
   --| Preconditions  : connected to the server
   --| Postconditions : disconnected from the server
   --| Exceptions     : None
   --| Note           : -
   ---------------------------------------------------------------------------
   procedure Disconnect( this : in out Object'Class );

   ---------------------------------------------------------------------------
   --| Description    :	Select a database at the server
   --| Preconditions  :	Connected to a server
   --| Postconditions :
   --| Exceptions     :
   --| Note           :
   ---------------------------------------------------------------------------
   procedure Select_DB( this : in out Object'Class; Name : in String ); 
   
   ---------------------------------------------------------------------------
   --| Description    : Perform an sql query. Input is a normal mysql query
   --|                  string.
   --| Preconditions  :
   --| Postconditions :
   --| Exceptions     :
   --| Note           :
   ---------------------------------------------------------------------------                                                             
   function Query( this: in Object'Class; query : in String ) 
                                                  return Query_ID;

   No_Query_Result  : exception;
 
   ---------------------------------------------------------------------------
   --| Description    : Drop the query
   --| Preconditions  :
   --| Postconditions :
   --| Exceptions     :
   --| Note           :
   ---------------------------------------------------------------------------                                                             
   procedure Drop_Query( this : in out Object'Class; id : in Query_ID );

   ---------------------------------------------------------------------------
   --| Description    : Perform an sql query.
   --| Preconditions  :
   --| Postconditions :
   --| Exceptions     :
   --| Note           :
   ---------------------------------------------------------------------------                
   procedure Next( this : in out Object'Class; query : in Query_ID);

   No_More_Rows : exception;

   ---------------------------------------------------------------------------
   --| Description    : Perform an sql query.
   --| Preconditions  :
   --| Postconditions :
   --| Exceptions     :
   --| Note           :
   ---------------------------------------------------------------------------  
   function Get_Field_Number( this : in Object; id : in Query_ID; name : in String )
                                                     return Field_Number;
						     

   ---------------------------------------------------------------------------
   --| Description    : Get the field of the current colume and return the 
   --|                  specified data type. Access via field number or a 
   --|                  field name is provided for all data types
   --| Preconditions  :				  
   --| Postconditions :
   --| Exceptions     :
   --| Note           :
   ---------------------------------------------------------------------------

   Unknown_Field     : exception;
   Field_Type_Error  : exception;
   Field_Parse_Error : exception;   
                   
   function String_Field( this : in Object; id : in Query_ID; name : in String ) 
                                                     return Unbounded_String;

				              
   function String_Field( this    : in Object; 
                          id      : in Query_ID; 
			  fNumber : in Field_Number ) 
                                                     return Unbounded_String;
            
   function Date_Field( this      : in Object; 
                        id        : in Query_ID; 
			fNumber   : in Field_Number ) return Time;
           
   function Date_Field( this      : in Object; 
                        id        : in Query_ID; 
			name      : in String ) return Time;
           
   function Blob_Field( this      : in Object; 
                        id        : in Query_ID; 
			fNumber   : in Field_Number ) return Blob;

           
   function Blob_Field( this      : in Object; 
                        id        : in Query_ID; 
			name      : in String ) return Blob;


   function Integer_Field( this      : in Object; 
                        id        : in Query_ID; 
			fNumber   : in Field_Number ) return Integer;

 
   function Integer_Field( this      : in Object; 
                        id        : in Query_ID; 
			name      : in String ) return Integer;

   function Float_Field( this      : in Object; 
                        id        : in Query_ID; 
			fNumber   : in Field_Number ) return Float;

 
   function Float_Field( this      : in Object; 
                        id        : in Query_ID; 
			name      : in String ) return Float;


   ---=====================================================================---
   ---===                   E X T E N S I O N                           ===---
   ---=====================================================================---
 

   ---=====================================================================---
private
   type Object_Data;
   type Object_Data_Access is access Object_Data;

   type Object is tagged record
         data : Object_Data_Access := null;
      end record; 

end DB;


